@@ -1,73 +1,29 @@
 """Dataclasses to use and configure planning and scheduling with."""
 
-import math
-from abc import ABC, abstractmethod
 from dataclasses import dataclass
-from typing import Iterable, Optional, Union
+from typing import Optional, Union
 
 from serde import ExternalTagging, field, serde
 
+from raplan import distributions
+
+__all__ = [
+    "Task",
+    "Maintenance",
+    "Component",
+    "System",
+    "Horizon",
+    "Project",
+    "ScheduleItem",
+    "Schedule",
+]
+
 
 def _is_empty(lst: Union[list, dict, set]) -> bool:
     """Check whether a list is empty."""
     return len(lst) == 0
 
 
-def compound_probability(probabilities: Iterable[float]) -> float:
-    """Compound CDF value of multiple contributions."""
-    return max(0.0, 1.0 - math.prod(1 - p for p in probabilities))
-
-
-class Distribution(ABC):
-    """Abstract base class for distributions."""
-
-    def cdf(self, x: Union[int, float] = 1.0) -> float:
-        """Cumulative probability density function of this distribution."""
-        if x < 0:
-            return 0.0
-        return self._cdf(x)
-
-    @abstractmethod
-    def _cdf(self, x: Union[int, float] = 1.0) -> float:
-        """Cumulative probability density function of this distribution."""
-
-
-@serde
-@dataclass
-class ContinuousDistribution(Distribution):
-    """A continuous rate distribution."""
-
-    rate: Union[int, float] = 1.0
-
-    def _cdf(self, x: Union[int, float] = 1.0) -> float:
-        return min(1.0, float(x * self.rate))
-
-
-@serde
-@dataclass
-class WeibullDistribution(Distribution):
-    """Weibull distribution (2-parameter).
-
-    Arguments:
-        alpha: Shape parameter.
-        mtbf: Mean time between failure.
-    """
-
-    alpha: Union[int, float] = 2.0
-    mtbf: Union[int, float] = 10.0
-
-    @property
-    def beta(self) -> float:
-        """Weibull scale parameter."""
-        return self.mtbf / math.gamma(1.0 + 1.0 / self.alpha)
-
-    def _cdf(self, x: Union[int, float] = 1.0) -> float:
-        return 1.0 - math.exp(-((x / self.beta) ** self.alpha))
-
-
-Distributions = Union[WeibullDistribution, ContinuousDistribution]
-
-
 @serde
 @dataclass
 class Task:
@@ -102,6 +58,11 @@ class Maintenance:
     task: Task = field(default_factory=Task)
     time: Union[int, float] = 1.0
 
+    @property
+    def end(self) -> Union[int, float]:
+        """End time of this maintenance."""
+        return self.time + self.task.duration
+
 
 @serde(tagging=ExternalTagging)
 @dataclass
@@ -118,7 +79,9 @@ class Component:
 
     name: Optional[str] = field(default=None, skip_if_default=True)
     age: Union[int, float] = 0.0
-    distribution: Distributions = field(default_factory=WeibullDistribution)
+    distribution: distributions.Distributions = field(
+        default_factory=distributions.Weibull
+    )
     maintenance: list[Maintenance] = field(default_factory=list, skip_if=_is_empty)
 
     def get_ordered_maintenance(self) -> list[Maintenance]:
@@ -172,7 +135,7 @@ class System:
         components' respective function incorporating maintenance.
         """
         if len(self.components):
-            return compound_probability(c.cfp(x) for c in self.components)
+            return distributions.compound_probability(c.cfp(x) for c in self.components)
         else:
             return 0.0
 
@@ -260,7 +223,7 @@ class Project:
         systems' respective function incorporating maintenance.
         """
         if len(self.systems):
-            return compound_probability(s.cfp(x) for s in self.systems)
+            return distributions.compound_probability(s.cfp(x) for s in self.systems)
         else:
             return 0.0
 
@@ -304,7 +267,7 @@ class Project:
         """Get a fully generated schedule."""
         return Schedule(
             tasks=[
-                ScheduledTask(
+                ScheduleItem(
                     m.task.name,
                     rejuvenation=m.task.rejuvenation,
                     duration=m.task.duration,
@@ -322,18 +285,19 @@ class Project:
 
 @serde
 @dataclass
-class ScheduledTask:
-    """A task with full detail regarding its system, component and maintenance info.
+class ScheduleItem:
+    """A schedule item with full detail regarding its system, component and maintenance
+    task info.
 
     Arguments:
-        name: Name for this action.
-        rejuvenation: Rejuvenation factor between [0.0-1.0]. Percentage of age that is
+        name: Name for this action. rejuvenation: Rejuvenation factor between [0.0-1.0].
+        Percentage of age that is
             regained. Therefore, 1.0 would mean a full replacement.
-        duration: Duration of the maintenance. Usually in years.
-        cost: Cost of the maintenance. Usually expressed in a currency or equivalent.
-        system: Name of the system to which this maintenance is applied.
-        component: Name of the component to which this maintenance is applied.
-        time: Time at which this maintenance is scheduled.
+        duration: Duration of the maintenance. Usually in years. cost: Cost of the
+        maintenance. Usually expressed in a currency or equivalent. system: Name of the
+        system to which this maintenance is applied. component: Name of the component to
+        which this maintenance is applied. time: Time at which this maintenance is
+        scheduled.
     """
 
     name: Optional[str] = field(default=None, skip_if_default=True)
@@ -354,8 +318,8 @@ class Schedule:
         tasks: Scheduled tasks.
     """
 
-    tasks: list[ScheduledTask] = field(default_factory=list, skip_if=_is_empty)
+    tasks: list[ScheduleItem] = field(default_factory=list, skip_if=_is_empty)
 
-    def get_ordered_maintenance(self) -> list[ScheduledTask]:
+    def get_ordered_maintenance(self) -> list[ScheduleItem]:
         """Get all tasks ordered in time."""
         return sorted(self.tasks, key=lambda t: t.time)
