@@ -3,7 +3,8 @@
 from dataclasses import dataclass
 from typing import Optional, Sequence, Union
 
-from raplan.classes import Component, Maintenance, Project, System, compound_probability
+from raplan.classes import Component, Maintenance, Project, System
+from raplan.distributions import compound_probability
 
 try:
     import plotly.graph_objects as go
